@@ -10,7 +10,7 @@
 
 #define PODOFO_VERSION_MAJOR 0
 #define PODOFO_VERSION_MINOR 9
-#define PODOFO_VERSION_PATCH 7
+#define PODOFO_VERSION_PATCH 8
 
 /* PoDoFo configuration options */
 #define PODOFO_MULTI_THREAD
@@ -42,7 +42,7 @@
 #define SZ_LONG 8
 
 /* Endianness */
-/* #undef TEST_BIG */
+#define TEST_BIG
 
 /* Features */
 /* #undef PODOFO_NO_FONTMANAGER */
