@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #ifndef _PDF_MEM_DOCUMENT_H_
@@ -23,6 +36,7 @@
 
 #include "podofo/base/PdfDefines.h"
 #include "podofo/base/PdfObject.h"
+#include "podofo/base/PdfExtension.h"
 
 #include "PdfDocument.h"
 #include "PdfFontCache.h"
@@ -70,34 +84,46 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     PdfMemDocument();
     
+    /** Construct a new (empty) PdfMemDocument
+     */
+    PdfMemDocument( bool bOnlyTrailer );
+
     /** Construct a PdfMemDocument from an existing PDF (on disk)
      *  \param pszFilename filename of the file which is going to be parsed/opened
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the pszFilename is copied
+     *  for later use by WriteUpdate.
      *  
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate
      */
-    PdfMemDocument( const char* pszFilename );
+    PdfMemDocument( const char* pszFilename, bool bForUpdate = false );
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200    // not for MS Visual Studio 6
 #else
     /** Construct a PdfMemDocument from an existing PDF (on disk)
      *  \param pszFilename filename of the file which is going to be parsed/opened
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the pszFilename is copied
+     *  for later use by WriteUpdate.
      *  
      *  This is an overloaded member function to allow working
-     *  with unicode characters. On Unix systes you can also path
+     *  with unicode characters. On Unix systems you can also path
      *  UTF-8 to the const char* overload.
      *
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate
      */
-    PdfMemDocument( const wchar_t* pszFilename );
+    PdfMemDocument( const wchar_t* pszFilename, bool bForUpdate = false );
 #endif
 #endif // _WIN32
 
@@ -108,68 +134,85 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
     /** Load a PdfMemDocument from a file
      *
      *  \param pszFilename filename of the file which is going to be parsed/opened
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the pszFilename is copied
+     *  for later use by WriteUpdate.
      *  
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate, LoadFromBuffer, LoadFromDevice
      */
-    void Load( const char* pszFilename );
+    void Load( const char* pszFilename, bool bForUpdate = false );
 
 #ifdef _WIN32
-#if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
-#else
     /** Load a PdfMemDocument from a file
      *
      *  \param pszFilename filename of the file which is going to be parsed/opened
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the pszFilename is copied
+     *  for later use by WriteUpdate.
      *  
      *  This is an overloaded member function to allow working
-     *  with unicode characters. On Unix systes you can also path
+     *  with unicode characters. On Unix systems you can also path
      *  UTF-8 to the const char* overload.
      *
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate, LoadFromBuffer, LoadFromDevice
      */
-    void Load( const wchar_t* pszFilename );
-#endif
+    void Load( const wchar_t* pszFilename, bool bForUpdate = false );
 #endif // _WIN32
 
     /** Load a PdfMemDocument from a buffer in memory
      *
      *  \param pBuffer a memory area containing the PDF data
      *  \param lLen length of the buffer
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the memory buffer is copied
+     *  for later use by WriteUpdate.
      *  
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate, Load, LoadFromDevice
      */
-    void Load( const char* pBuffer, long lLen );
+    void LoadFromBuffer( const char* pBuffer, long lLen, bool bForUpdate = false );
 
     /** Load a PdfMemDocument from a PdfRefCountedInputDevice
      *
      *  \param rDevice the input device containing the PDF
+     *  \param bForUpdate whether to load for incremental update
      *
      *  This might throw a PdfError( ePdfError_InvalidPassword ) exception
      *  if a password is required to read this PDF.
      *  Call SetPassword with the correct password in this case.
+     *
+     *  When the bForUpdate is set to true, the rDevice is referenced
+     *  for later use by WriteUpdate.
      *  
-     *  \see SetPassword
+     *  \see SetPassword, WriteUpdate, Load, LoadFromBuffer
      */
-    void Load( const PdfRefCountedInputDevice & rDevice );
+    void LoadFromDevice( const PdfRefCountedInputDevice & rDevice, bool bForUpdate = false );
+
+    /** Returns whether the document is fully loaded.
+     */
+    inline bool IsLoaded( void ) const;
 
     /** Writes the complete document to a file
      *
      *  \param pszFilename filename of the document 
      *
-     *  \see Write
+     *  \see Write, WriteUpdate
      *
-     *  This is an overloaded member function for your convinience.
+     *  This is an overloaded member function for your convenience.
      */
     void Write( const char* pszFilename );
 
@@ -178,13 +221,13 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      *
      *  \param pszFilename filename of the document 
      *
-     *  \see Write
+     *  \see Write, WriteUpdate
      *
      *  This is an overloaded member function to allow working
-     *  with unicode characters. On Unix systes you can also path
+     *  with unicode characters. On Unix systems you can also path
      *  UTF-8 to the const char* overload.
      *
-     *  This is an overloaded member function for your convinience.
+     *  This is an overloaded member function for your convenience.
      */
     void Write( const wchar_t* pszFilename );
 #endif // _WIN32
@@ -192,9 +235,78 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
     /** Writes the complete document to an output device
      *
      *  \param pDevice write to this output device
+     *
+     *  \see WriteUpdate
      */
     void Write( PdfOutputDevice* pDevice );
 
+    /** Writes the document changes to a file
+     *
+     *  \param pszFilename filename of the document 
+     *
+     *  Writes the document changes to a file as an incremental update.
+     *  The document should be loaded with bForUpdate = true, otherwise
+     *  an exception is thrown.
+     *
+     *  Beware when overwriting existing files. Plain file overwrite is allowed
+     *  only if the document was loaded with the same filename (and the same overloaded
+     *  function), otherwise the destination file cannot be the same as the source file,
+     *  because the destination file is truncated first and only then the source file
+     *  content is copied into it.
+     *
+     *  \see Write, WriteUpdate
+     *
+     *  This is an overloaded member function for your convenience.
+     */
+    void WriteUpdate( const char* pszFilename );
+
+#ifdef _WIN32
+    /** Writes the document changes to a file
+     *
+     *  \param pszFilename filename of the document 
+     *
+     *  Writes the document changes to a file as an incremental update.
+     *  The document should be loaded with bForUpdate = true, otherwise
+     *  an exception is thrown.
+     *
+     *  Beware when overwriting existing files. Plain file overwrite is allowed
+     *  only if the document was loaded with the same filename (and the same overloaded
+     *  function), otherwise the destination file cannot be the same as the source file,
+     *  because the destination file is truncated first and only then the source file
+     *  content is copied into it.
+     *
+     *  \see Write, WriteUpdate
+     *
+     *  This is an overloaded member function to allow working
+     *  with unicode characters. On Unix systems you can also path
+     *  UTF-8 to the const char* overload.
+     *
+     *  This is an overloaded member function for your convenience.
+     */
+    void WriteUpdate( const wchar_t* pszFilename );
+#endif // _WIN32
+
+    /** Writes the document changes to an output device
+     *
+     *  \param pDevice write to this output device
+     *  \param bTruncate whether to truncate the pDevice first and fill it
+     *                   with the content of the source document; the default is true.
+     *
+     *  Writes the document changes to the output device as an incremental update.
+     *  The document should be loaded with bForUpdate = true, otherwise
+     *  an exception is thrown.
+     *
+     *  The bTruncate is used to determine whether saving to the same file or not.
+     *  In case the bTruncate is true, a new source stream is opened and its whole
+     *  content is copied to the pDevice first. Otherwise the pDevice is the same
+     *  file which had been loaded and the caller is responsible to position
+     *  the pDevice at the place, where the update should be written (basically
+     *  at the end of the stream).
+     *
+     *  \see Write, WriteUpdate
+     */
+    void WriteUpdate( PdfOutputDevice* pDevice, bool bTruncate = true );
+
     /** Set the write mode to use when writing the PDF.
      *  \param eWriteMode write mode
      */
@@ -215,6 +327,31 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      *  \returns EPdfVersion version of the pdf document
      */
     EPdfVersion GetPdfVersion() const { return m_eVersion; }
+    
+    /** Add a vendor-specific extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    void AddPdfExtension( const char* ns, pdf_int64 level );
+    
+    /** Checks whether the documents is tagged to imlpement a vendor-specific 
+     *  extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    bool HasPdfExtension( const char* ns, pdf_int64 level ) const;
+    
+    /** Remove a vendor-specific extension to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    void RemovePdfExtension( const char* ns, pdf_int64 level );
+    
+    /** Return the list of all vendor-specific extensions to the current PDF version.
+     *  \param ns  namespace of the extension
+     *  \param level  level of the extension
+     */
+    std::vector<PdfExtension> GetPdfExtensions() const;
 
     /** If you try to open an encrypted PDF file, which requires
      *  a password to open, PoDoFo will throw a PdfError( ePdfError_InvalidPassword ) 
@@ -256,7 +393,7 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
                                         PdfEncrypt::ePdfPermissions_Accessible |
                                         PdfEncrypt::ePdfPermissions_DocAssembly |
                                         PdfEncrypt::ePdfPermissions_HighPrint,
-                       PdfEncrypt::EPdfEncryptAlgorithm eAlgorithm = PdfEncrypt::ePdfEncryptAlgorithm_RC4V1,
+                       PdfEncrypt::EPdfEncryptAlgorithm eAlgorithm = PdfEncrypt::ePdfEncryptAlgorithm_AESV2,
                        PdfEncrypt::EPdfKeyLength eKeyLength = PdfEncrypt::ePdfKeyLength_40 );
 
     /** Encrypt the document during writing using a PdfEncrypt object
@@ -472,7 +609,8 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
      */
     inline const PdfEncrypt* GetEncrypt() const;
 
- private:
+private:
+
     /** Get a dictioary from the catalog dictionary by its name.
      *  \param pszName will be converted into a PdfName
      *  \returns the dictionary if it was found or NULL
@@ -511,8 +649,27 @@ class PODOFO_DOC_API PdfMemDocument : public PdfDocument {
 
     PdfParser*      m_pParser; ///< This will be temporarily initialized to a PdfParser object so that SetPassword can work
     EPdfWriteMode   m_eWriteMode;
+
+    bool m_bSoureHasXRefStream;
+    EPdfVersion m_eSourceVersion;
+    pdf_int64 m_lPrevXRefOffset;
+#ifdef _WIN32
+    wchar_t *m_wchar_pszUpdatingFilename;
+#endif
+    char *m_pszUpdatingFilename;
+    PdfRefCountedInputDevice *m_pUpdatingInputDevice;
+
+    inline bool IsLoadedForUpdate( void ) const;
 };
 
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+bool PdfMemDocument::IsLoaded( void ) const
+{
+    return m_pParser == NULL;
+}
+
 // -----------------------------------------------------
 // 
 // -----------------------------------------------------
@@ -585,6 +742,18 @@ const PdfEncrypt* PdfMemDocument::GetEncrypt() const
     return m_pEncrypt; 
 }
 
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+bool PdfMemDocument::IsLoadedForUpdate( void ) const
+{
+    return m_pszUpdatingFilename ||
+#ifdef _WIN32
+        m_wchar_pszUpdatingFilename ||
+#endif
+        m_pUpdatingInputDevice;
+}
+
 };
 
 
